<?php

namespace Drupal\simple_sitemap_auth\Form;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_sitemap\Simplesitemap;
use Drupal\simple_sitemap\Form\FormHelper;
use Drupal\simple_sitemap\Form\SimplesitemapFormBase;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * SimplesitemapForm for Auth.
 *
 * @package Drupal\simple_sitemap_auth\Form
 */
class SimplesitemapauthSettingsForm extends SimplesitemapFormBase {

  /**
   * Language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The variable containing the user manager.
   *
   * @var \Drupal\user\UserStorage
   */
  protected $userManager;

  /**
   * SimplesitemapSettingsForm constructor.
   *
   * @param \Drupal\simple_sitemap\Simplesitemap $generator
   *   Sitemap generator.
   * @param \Drupal\simple_sitemap\Form\FormHelper $form_helper
   *   Form helper.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   Entity type manager.
   */
  public function __construct(
    Simplesitemap $generator,
    FormHelper $form_helper,
    LanguageManagerInterface $language_manager,
    EntityTypeManager $entityTypeManager
  ) {
    parent::__construct(
      $generator,
      $form_helper
    );
    $this->languageManager = $language_manager;
    $this->userManager = $entityTypeManager->getStorage('user');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('simple_sitemap.generator'),
      $container->get('simple_sitemap.form_helper'),
      $container->get('language_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simple_sitemap_auth_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['config.doccheck_basic'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('simple_sitemap_auth.settings');
    $form['simple_sitemap_auth_settings']['user'] = [
      '#type' => 'select',
      '#title' => $this->t('User'),
      '#description' => $this->t('The user whose permissions are used for crawling. Anonymous user (default) and all users except users with administrator role.'),
      '#default_value' => $config->get('user'),
      '#required' => TRUE,
      '#options' => $this->userList(),
    ];

    $this->formHelper->displayRegenerateNow($form['simple_sitemap_settings']);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->getEditable('simple_sitemap_auth.settings');
    $config->set('user', $form_state->getValue('user'));
    $config->save();

    parent::submitForm($form, $form_state);

    // Regenerate sitemaps according to user setting.
    if ($form_state->getValue('simple_sitemap_regenerate_now')) {
      $this->generator->setVariants(TRUE)
        ->rebuildQueue()
        ->generateSitemap();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function userList() {
    $filtered_users = [];
    $user_query = $this->userManager->getQuery();

    // Users with explicit roles set except administrators.
    $role_user = $user_query
      ->andConditionGroup()
      ->exists('roles')
      ->condition('status', 1)
      ->condition('roles', 'administrator', '<>');

    // Users with simple authenticated role.
    $auth_user = $user_query
      ->andConditionGroup()
      ->notExists('roles')
      ->condition('status', 1);

    // All groups above and anonymous user.
    $user_group = $user_query
      ->orConditionGroup()
      ->condition('uid', 0)
      ->condition($auth_user)
      ->condition($role_user);
    $ids = $this->userManager->getQuery()
      ->condition($user_group)
      ->execute();
    $all_users = $this->userManager->loadMultiple($ids);
    /** @var \Drupal\Core\Session\AccountInterface $value */
    foreach ($all_users as $key => $value) {
      $filtered_users[$key] = $value->getDisplayName();
    }
    return $filtered_users;
  }

}
