<?php

namespace Drupal\simple_sitemap_auth\Plugin\simple_sitemap\UrlGenerator;

use Drupal\simple_sitemap\EntityHelper;
use Drupal\simple_sitemap\Logger;
use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\EntityUrlGenerator;
use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\UrlGeneratorManager;
use Drupal\simple_sitemap\Simplesitemap;
use Drupal\Core\Cache\MemoryCache\MemoryCacheInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * EntityUrlGenerator for Auth.
 *
 * @package Drupal\simple_sitemap_auth\Plugin\simple_sitemap\UrlGenerator
 *
 * @UrlGenerator(
 *   id = "authentity",
 *   label = @Translation("Auth Entity URL generator"),
 *   description = @Translation("Generates URLs for entity bundles and bundle overrides of an authenticated user."),
 * )
 */
class AuthEntityUrlGenerator extends EntityUrlGenerator {

  /**
   * AuthEntityUrlGenerator constructor.
   *
   * @param array $configuration
   *   Configuration.
   * @param string $plugin_id
   *   PluginID.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\simple_sitemap\Simplesitemap $generator
   *   Sitemap generator.
   * @param \Drupal\simple_sitemap\Logger $logger
   *   Logger.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Drupal\simple_sitemap\EntityHelper $entityHelper
   *   Entity helper.
   * @param \Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\UrlGeneratorManager $url_generator_manager
   *   Url generator manager.
   * @param \Drupal\Core\Cache\MemoryCache\MemoryCacheInterface $memory_cache
   *   Memory cache.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    Simplesitemap $generator,
    Logger $logger,
    LanguageManagerInterface $language_manager,
    EntityTypeManagerInterface $entity_type_manager,
    EntityHelper $entityHelper,
    UrlGeneratorManager $url_generator_manager,
    MemoryCacheInterface $memory_cache
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $generator,
      $logger,
      $language_manager,
      $entity_type_manager,
      $entityHelper,
      $url_generator_manager,
      $memory_cache
    );
    $this->anonUser = $this->getUser();
  }

  /**
   * AuthEntityUrlGenerator creator.
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('simple_sitemap.generator'),
      $container->get('simple_sitemap.logger'),
      $container->get('language_manager'),
      $container->get('entity_type.manager'),
      $container->get('simple_sitemap.entity_helper'),
      $container->get('plugin.manager.simple_sitemap.url_generator'),
      $container->get('entity.memory_cache')
    );
  }

  /**
   * Set user access for generated links.
   *
   * @return \Drupal\user\Entity\User|\Drupal\Core\Entity\EntityInterface|null
   *   User object.
   */
  protected function getUser() {
    $user = $this->anonUser;
    $config = \Drupal::config('simple_sitemap_auth.settings');
    if ($config->get("user") && User::load($config->get("user"))) {
      $user = User::load($config->get("user"));
    }
    return $user;
  }

}
