<?php

namespace Drupal\Tests\simple_sitemap_auth\Functional;

use Drupal\simple_sitemap\Queue\QueueWorker;
use Drupal\Tests\simple_sitemap\Functional\SimplesitemapTestBase;

/**
 * Tests Simple XML Sitemap functional integration.
 *
 * @group simple_sitemap_auth
 */
class SimplesitemapAuthTest extends SimplesitemapTestBase {

  /**
   * Modules to enable for this test.
   *
   * @var string[]
   */
  protected static $modules = [
    'simple_sitemap',
    'simple_sitemap_auth',
    'node',
    'node_view_permissions',
    'content_translation',
  ];

  /**
   * Auth user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * A node.
   *
   * @var \Drupal\node\Entity\Node
   */
  protected $node3;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->user = $this->drupalCreateUser([], NULL, FALSE);
    $this->drupalLogin($this->privilegedUser);

    $this->drupalCreateContentType(['type' => 'article']);

    // Set sitemap user.
    $this->drupalPostform('/admin/config/search/simplesitemap/user', [
      'user' => $this->user->id(),
    ], 'Save configuration');

    // Set auth variant as default.
    $this->drupalPostform('/admin/config/search/simplesitemap/variants', [
      'variants' => 'default | auth_hreflang | Default',
    ], 'Save configuration');

    // Set default as default.
    $this->drupalPostform('/admin/config/search/simplesitemap/settings', [
      'default_variant' => 'default',
    ], 'Save configuration');

    // Set nodetype sitemap index.
    $this->drupalPostform('/admin/structure/types/manage/page', [
      'index_default_node_settings' => TRUE,
    ], 'Save content type');

    $this->drupalPostform('/admin/structure/types/manage/article', [
      'index_default_node_settings' => TRUE,
    ], 'Save content type');

    // Set permissions.
    $this->drupalPostform('/admin/people/permissions', [
      'anonymous[view any page content]' => TRUE,
      'authenticated[view any article content]' => TRUE,
    ], 'Save permissions');
    $this->drupalPostForm('admin/reports/status/rebuild', [], 'Rebuild permissions');

    // Set permissions.
    $this->node3 = $this->createNode(['title' => 'Node3', 'type' => 'article']);

    // Generate sitemap.
    $this->generator->generateSitemap(QueueWorker::GENERATE_TYPE_BACKEND);
  }

  /**
   * Tests Auth Sitemap.
   */
  public function testSitemapAuth() {
    $this->drupalGet($this->defaultSitemapUrl);
    $this->assertSession()->responseContains('node/' . $this->node->id());
    $this->assertSession()->responseContains('node/' . $this->node2->id());
    $this->assertSession()->responseContains('node/' . $this->node3->id());
  }

}
